@@ -10,7 +10,9 @@ from pairtree import FileNotFoundException, ObjectNotFoundException
 
 from datetime import datetime
 
-from os import path, mkdir, rename
+from os import mkdir, rename
+
+import os
 
 from shutil import copy2
 
@@ -94,7 +96,7 @@ class HarvestedRecord(object):
         self.manifest['metadata_files'][version] = []
         self.manifest['files'][version] = []
         self.set_version_date(version, date)
-        self.po.add_bytestream_by_path(path.join("__"+str(version), "4=%s" % id_encode(self.item_id)), self.item_id)
+        self.po.add_bytestream_by_path(os.path.join("__"+str(version), "4=%s" % id_encode(self.item_id)), self.item_id)
 
     def _init_manifests_emptydatastructures(self):
         self.item_id = self.manifest['item_id']
@@ -117,7 +119,7 @@ class HarvestedRecord(object):
             # init from disc
             for filename in [x for x in self.po.list_parts("__"+str(version)) if not (x.startswith("0=") or x.startswith("1=") or x.startswith("2=") or x.startswith("3=") or x.startswith("4=") or x.startswith("5=")) ]:
                 logger.debug("Item %s has file: %s" % (self.item_id, x) )
-                if self.po.isdir(path.join("__"+str(version), filename)):
+                if self.po.isdir(os.path.join("__"+str(version), filename)):
                     self.manifest['subdir'][version].append(filename)
                 self.manifest['files'][version].append(filename)
 
@@ -168,7 +170,7 @@ class HarvestedRecord(object):
         return self.po.fs._id_to_dirpath(self.po.id)
 
     def revert(self, **kw):
-        if not path.isdir(self.itempath):
+        if not os.path.isdir(self.itempath):
             logger.error("Path to harvested item does not exist")
             raise Exception("Path to harvested item does not exist")
         try:
@@ -216,7 +218,7 @@ class HarvestedRecord(object):
             self.manifest['metadata_files'][version].append(filename)
         if filename not in self.manifest['files'][version]:
             self.manifest['files'][version].append(filename)
-        resp = self.po.add_bytestream_by_path(path.join("__" + str(version), filename), filetostream)
+        resp = self.po.add_bytestream_by_path(os.path.join("__" + str(version), filename), filetostream)
         self._reload_filelist(version)
         if sync:
             self.sync()
@@ -227,7 +229,7 @@ class HarvestedRecord(object):
         if not version:
             version = self.manifest['currentversion']
         if self.isfile(filename, version):
-            return self.po.get_bytestream_by_path(path.join("__" + str(version), filename),
+            return self.po.get_bytestream_by_path(os.path.join("__" + str(version), filename),
                                                   streamable=True, appendable=writeable)
         else:
             raise FileNotFoundException
@@ -237,7 +239,7 @@ class HarvestedRecord(object):
             versions = [self.manifest['currentversion']]
         for version in versions:
             try:
-                self.po.del_file_by_path(path.join("__" + str(version), filename))
+                self.po.del_file_by_path(os.path.join("__" + str(version), filename))
                 if self.isfile(filename, version):
                     self.manifest['metadata_files'][version].remove(filename)
                 self._reload_filelist(version)
@@ -248,16 +250,29 @@ class HarvestedRecord(object):
     def isfile(self, filepath, version=None):
         if not version:
             version = self.manifest['currentversion']
-        return self.po.isfile(path.join("__"+str(version), filepath))
+        return self.po.isfile(os.path.join("__"+str(version), filepath))
 
     def isdir(self, filepath, version=None):
         if not version:
             version = self.manifest['currentversion']
-        return self.po.isdir(path.join("__"+str(version),filepath))
+        return self.po.isdir(os.path.join("__"+str(version),filepath))
 
     def list_parts(self, subpath=""):
         if self.isdir(subpath):
-            return self.po.list_parts(path.join("__"+str(self.manifest['currentversion']),subpath))
+            return self.po.list_parts(os.path.join("__"+str(self.manifest['currentversion']),subpath))
+    
+    def to_dirpath(self, filepath=None, version=None):
+        if not version:
+            version = self.currentversion
+        if filepath:
+            return os.path.join(ppath.id_to_dirpath(self.po.id, self.po.fs.pairtree_root), "__%s" % version, filepath)
+        else:
+            return os.path.join(ppath.id_to_dirpath(self.po.id, self.po.fs.pairtree_root), "__%s" % version)
+
+    def stat(self, filepath, version=None):
+        if not version:
+            version = self.currentversion
+        return self.po.stat(filepath)
 
     def set_version_cursor(self, version):
         if version in self.manifest['versions']:
@@ -285,6 +300,27 @@ class HarvestedRecord(object):
         self.sync()
         return new_version
 
+    def move_directory_as_new_version(self, src_directory, version=None, force=False, date=None, _sync=True):
+        if not date:
+            date = datetime.now().isoformat()
+        self.manifest['date'] = date
+        if not version:
+            version = self._incr_version(self.currentversion)
+        if version in self.get_versions():
+            if force:
+                self.del_version(version)
+            else:
+                raise Exception("Cannot move a directory onto an already existing version")
+        version_path = os.path.join(ppath.id_to_dirpath(self.po.id, self.po.fs.pairtree_root), "__%s" % version)
+        os.rename(src_directory, version_path)
+        self._setup_version_dir(version, date)
+        self._read_date()
+        self._reload_filelist(version)
+        self.set_version_cursor(version)
+        if _sync:
+            self.sync()
+        return version
+
     def clone_version(self, original_version, new_version, exclude_filenames=[]):
         if original_version in self.manifest['versions']:
             date = self.manifest['version_dates'][original_version]
@@ -308,7 +344,7 @@ class HarvestedRecord(object):
             self.manifest['version_dates'][new_name] = self.manifest['version_dates'][original_version]
             self.manifest['files'][new_name] = self.manifest['files'][original_version]
             self.manifest['metadata_files'][new_name] = self.manifest['metadata_files'][original_version]
-            rename(path.join(self.path_to_item(), "__"+str(original_version)), path.join(self.path_to_item(), "__" + str(new_name)))
+            rename(os.path.join(self.path_to_item(), "__"+str(original_version)), os.path.join(self.path_to_item(), "__" + str(new_name)))
             self.set_version_cursor(new_name)
             self.manifest['versions'].remove(original_version)
             del self.manifest['version_dates'][original_version]
@@ -359,7 +395,7 @@ class HarvestedRecord(object):
     def set_version_date(self, version, date):
         if version in self.manifest['versions']:
             self.manifest['version_dates'][version] = date
-            self.po.add_bytestream_by_path(path.join("__"+str(version), "3=%s" % id_encode(date)), date)
+            self.po.add_bytestream_by_path(os.path.join("__"+str(version), "3=%s" % id_encode(date)), date)
             return True
         else:
             logger.error("Version %s does not exist" % version)
@@ -382,7 +418,7 @@ class RDFRecord(HarvestedRecord):
         if not version:
             version = self.get_version_cursor()
         fn = self.manifest.get('rdffilename', 'manifest.rdf')
-        return path.join(self.po.fs._id_to_dirpath(self.po.id), "__"+str(version), fn)
+        return os.path.join(self.po.fs._id_to_dirpath(self.po.id), "__"+str(version), fn)
 
     def load_rdf_manifest(self, version=None):
         format = self.manifest.get('rdffileformat', 'xml')
@@ -414,6 +450,12 @@ class RDFRecord(HarvestedRecord):
     ##############
     ## Classes to annotate with rdf manifest updating
     
+    def put_stream(self, filename, filetostream, version=None, metadata=False, sync=True):
+        super(RDFRecord, self).put_stream(self, filename, filetostream, version=version, metadata=metadata, sync=False)
+        self.add_triple(self.uri, "ore:aggregates", "%s/%s" % (self.uri, filename))
+        if sync:
+            self.sync()
+    
     def set_version_cursor(self, version):
         super(RDFRecord, self).set_version_cursor(version)
         self.load_rdf_manifest()
@@ -434,3 +476,9 @@ class RDFRecord(HarvestedRecord):
     def _copy_version(self, latest_version, new_version, exclude_filenames=[]):
         super(RDFRecord, self)._copy_version(latest_version, new_version, exclude_filenames)
         self.load_rdf_manifest()
+    
+    def move_directory_as_new_version(self, src_directory, version=None, force=False, date=None, _sync=True):
+        super(RDFRecord, self).move_directory_as_new_version(src_directory, version=version, force=force, date=date, _sync=False)
+        self.load_rdf_manifest()
+        if _sync:
+            self.sync()
